function p=z_read_seismograms

addpath /l1/heiner/Bin


close all
clear

% function p=z_read_seismograms(array,spacing)
%
% reads seismograms out of the output files and plots them
%
% INPUT:
%
% array:    array of the indeces of the seismograms that will be plotted,
%           enter [] if all seismograms are to be plotted
%
% spacing:  spacing of seismograms in the plot as multiple of the maximum
%           amplitude. A reasonable value is 2.0 .
%
% OUTPUT:   a seismogram structure
%
% last modified: 30 June, 2006


% Simplue GUI with defaults
dlg_title = 'Parameters for plotting';
prompt = {'Number of processors','Corner frequency (Hz)','Filter order', 'Normalization?','Seismogram spacing','Source (x) ','Source (y)','Sort for distance?'};
                    num_lines=1;
                    def ={'64','1','2','1','1','90','0','1'};
                    answer  = inputdlg(prompt,dlg_title,num_lines,def);                   
                    np= str2num(char(answer(1)));
                    fup=str2num(char(answer(2)));
                    ord=str2num(char(answer(3)));
                    inorm=str2num(char(answer(4)));
                    spacing=str2num(char(answer(5)));                    
                    sx=str2num(char(answer(6)));
                    sy=str2num(char(answer(7))); 
                    isort=str2num(char(answer(8)));
                    
% End GUI


%np=input(' Give number of processors ');
%spacing=input(' Spacing (e.g. 2) ');
%fup=input(' Give corner frequency ');
%ord=input(' Give order of filter (2) ');
%inorm=input(' Normalize each trace (0-1) ?')

for ic=1:3,
    
    if ic==1, component='theta';end    
    if ic==2, component='phi';end    
    if ic==3, component='r';end
    
    
%component=input('component (theta, phi, r): ','s');



% loop over all processors

ir_global=0;

for i=1:np

processor=sprintf('%i',i);

filename=['../DATA/OUTPUT/seismogram_' component processor];

fid=fopen(filename,'r');

if fid>0,

fgets(fid);
fscanf(fid,'%c',4);
p.nr=fscanf(fid,'%d',1);
fscanf(fid,'%c',5);
p.nt=fscanf(fid,'%d',1);
fscanf(fid,'%c',5);
p.dt=fscanf(fid,'%g',1);

%p.r_theta=zeros(1,p.nr);
%p.r_phi=zeros(1,p.nr);
%p.seismograms=zeros(p.nr,p.nt);

p.t0=0.0;

for ir=1:p.nr
    
    ir_global=ir_global+1;
    fscanf(fid,'%c',9);
    p.r_theta(ir_global)=fscanf(fid,'%g',1);
    p.r_theta(ir_global)=p.r_theta(ir_global)*180/pi;
    fscanf(fid,'%c',6);
    p.r_phi(ir_global)=fscanf(fid,'%g',1);
    p.r_phi(ir_global)=p.r_phi(ir_global)*180/pi;
    p.r_dist(ir_global)=sqrt((p.r_theta(ir_global)-sx)^2+(p.r_phi(ir_global)-sy)^2);
    
    
    for it=1:p.nt    
        p.seismograms(ir_global,it)=fscanf(fid,'%g',1);
    end
    p.seismograms(ir_global,:)=lowpass(p.seismograms(ir_global,:),p.dt,fup,ord);
    if inorm==1,p.seismograms(ir_global,:)=p.seismograms(ir_global,:)/max(p.seismograms(ir_global,:));end
end

end

end

p.nr=ir_global;

if isort==1,
    [junk ii]=sort(p.r_dist);
end


disp(sprintf(' Found %i seismograms, now plotting ... ',p.nr));

% plot seismograms

m=max(max(abs(p.seismograms)));
t=0:p.dt:((p.nt-1)*p.dt);

figure
hold on


    for ir=1:p.nr
        plot(t,ir*spacing*m+p.seismograms(ii(ir),:),'b');
        text(1.05*t(end),ir*spacing*m,num2str(ii(ir)));
        %text(-0.09*t(end),ir*spacing*m+0.1*m,num2str(p.r_theta(ir)));
        %text(-0.01*t(end),ir*spacing*m+0.1*m,num2str(p.r_phi(ir)));
    end

    axis([0*t(end) 1.2*t(end) 0 p.nr*spacing*m+1.1*m]);
    grid on
    
title(sprintf('%s, f_cut=%g Hz',component,fup))
xlabel('time [s]');
ylabel('velocity [m/s]');


end

% Plot station distribution

figure
hold on

for i=1:p.nr  
    plot(p.r_theta(i),p.r_phi(i),'+'),
    text(p.r_theta(i),p.r_phi(i),sprintf('%i',i))   
end
    plot(sx,sy,'o')   
    text(sx,sy,'Epicentre') 
    xlabel(' Theta (deg) ')
    ylabel(' Phi (deg) ')
    title(' Receiver locations ')
    

