program make_slice_gradient
implicit none

        !========================================================
        ! local variables
        !========================================================

        integer :: par, plane, dummy, proc, it
        integer :: i, j, k, index, status
        integer :: nx, ny, nz
        integer :: nx_global, ny_global, nz_global
        integer :: pp, px, py, pz

        integer :: ix_multi, iy_multi, iz_multi
        integer :: nx_min, nx_max, ny_min, ny_max, nz_min, nz_max

        real :: val, dist, real_dummy
        real :: xmin, xmax, ymin, ymax, zmin, zmax
        real :: xmin_global, xmax_global, ymin_global, ymax_global, zmin_global, zmax_global
        real, allocatable, dimension(:) :: xco, yco, zco
        real, allocatable, dimension(:,:,:) :: model
        real, allocatable, dimension(:,:) :: slice

        character(len=50) :: junk, cit

        !========================================================
        ! read model parameter file
        !========================================================

        open(unit=10,file='MODELS/model_parameters',action='read')

        read(10,*) junk
        read(10,*) nx_global
        read(10,*) ny_global
        read(10,*) nz_global
        
        read(10,*) xmin_global
        read(10,*) xmax_global
        read(10,*) ymin_global
        read(10,*) ymax_global
        read(10,*) zmin_global
        read(10,*) zmax_global

        read(10,*) junk
        read(10,*) dummy
        read(10,*) real_dummy
        read(10,*) real_dummy
        read(10,*) real_dummy
        read(10,*) real_dummy
        read(10,*) real_dummy
        read(10,*) real_dummy
        read(10,*) dummy
        read(10,*) dummy
        read(10,*) junk

        read(10,*) px
        read(10,*) py
        read(10,*) pz

        close(unit=10)

        write(*,*) '============================================'
        write(*,*) ' geometrical model parameters =============='
        write(*,*) '============================================'
        write(*,*) xmin_global, '<= x <=', xmax_global
        write(*,*) ymin_global, '<= y <=', ymax_global
        write(*,*) zmin_global, '<= z <=', zmax_global
        write(*,*) '============================================'
        write(*,*) 'processors (px,py,pz):', px, py, pz
        write(*,*) '============================================'

        !========================================================
        ! read input
        !========================================================

        write(*,*) 'gradient (rho=1,lambda=2,mu=3): '
        read(*,*) par

        write(*,*) 'plane (x=1, y=2, z=3): '
        read(*,*) plane

        write(*,*) 'coordinate value: '
        read(*,*) val

        write(*,*) 'iteration: '
        read(*,*) it
        call int2str(it,cit)

        !========================================================
        ! allocate memory for coordinates and slices
        !========================================================

        if (plane==1) then

           allocate(slice(0:ny_global,0:nz_global),STAT=status)
           
        elseif (plane==2) then
           
           allocate(slice(0:nx_global,0:nz_global),STAT=status)

        elseif (plane==3) then

           allocate(slice(0:nx_global,0:ny_global),STAT=status)

        endif

        allocate(xco(0:nx_global),STAT=status)
        allocate(yco(0:ny_global),STAT=status)
        allocate(zco(0:nz_global),STAT=status)

        !========================================================
        ! read boxfile
        !========================================================

       	open(UNIT=15,FILE='MODELS/boxfile',STATUS='OLD',ACTION='READ')
	
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
		
	read(15,*) pp
	read(15,*) px
	read(15,*) py
	read(15,*) pz

	read(15,*) junk
		
	do k=1,pp
			
                read(15,*) proc
		read(15,*) ix_multi, iy_multi, iz_multi
		read(15,*) nx_min, nx_max
		read(15,*) ny_min, ny_max
		read(15,*) nz_min, nz_max
		read(15,*) xmin, xmax
		read(15,*) ymin, ymax
		read(15,*) zmin, zmax
		read(15,*) junk

                !==========================================================================
                ! find relevant processors
                !==========================================================================

                if ((plane==1) .and. (xmin<=val) .and. (xmax>=val)) then
                   
                   !=======================================================================
                   ! make coordinate files
                   !=======================================================================

                   call int2str(proc-1,junk)

                   if (iz_multi==1) then
                      open(unit=12,file='DATA/COORDINATES/yco_'//junk(1:len_trim(junk)),action='read')
                      read(12,*) junk
                      read(12,*) ny
                      read(12,*) junk
                      do i=0,ny-1
                         read(12,*) yco(ny_min+i)
                      enddo
                      close(unit=12)
                   endif

                   call int2str(proc-1,junk)

                   if (iy_multi==1) then
                      open(unit=13,file='DATA/COORDINATES/zco_'//junk(1:len_trim(junk)),action='read')
                      read(13,*) junk
                      read(13,*) nz
                      read(13,*) junk
                      do i=0,nz-1
                         read(13,*) zco(nz_max-i)
                      enddo
                      close(unit=13)
                   endif

                   !=======================================================================
                   ! find index
                   !=======================================================================
                   
                   call int2str(proc-1,junk)

                   open(unit=13,file='DATA/COORDINATES/xco_'//junk(1:len_trim(junk)),action='read')
                   read(13,*) junk
                   read(13,*) nx
                   read(13,*) junk
                   do i=0,nx-1
                      read(13,*) xco(i)
                   enddo
                   close(unit=13)

                   dist=xmax_global
                   index=0
                   
                   do i=0,(nx_max-nx_min)
                      if (abs(xco(i)-val)<dist) then
                         index=i
                         dist=abs(xco(i)-val)
                      endif
                   enddo
                   
                   !=======================================================================
                   ! make slice
                   !=======================================================================

                   call int2str(proc-1,junk)

                   if (par==1) then
                      open(unit=1001,file='DATA/OUTPUT/grad_rho__'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   elseif (par==2) then
                      open(unit=1001,file='DATA/OUTPUT/grad_lambda_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   elseif (par==3) then
                      open(unit=1001,file='DATA/OUTPUT/grad_mu_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   endif

                   allocate(model(nx_min:nx_max,ny_min:ny_max,nz_min:nz_max),STAT=status)
                   read(1001) model
                   
                   do i=ny_min,ny_max
                      do j=nz_min,nz_max
                         slice(i,j)=model(index+nx_min,i,nz_max+nz_min-j)
                      enddo
                   enddo

                   close(unit=1001)
                   deallocate(model)

                   !=======================================================================
                   !=======================================================================
                      

                elseif ((plane==2) .and. (ymin<=val) .and. (ymax>=val)) then


                   !=======================================================================
                   ! make coordinate files
                   !=======================================================================

                   call int2str(proc-1,junk)

                   if (iz_multi==1) then
                      open(unit=12,file='DATA/COORDINATES/xco_'//junk(1:len_trim(junk)),action='read')
                      read(12,*) junk
                      read(12,*) nx
                      read(12,*) junk
                      do i=0,nx-1
                         read(12,*) xco(nx_min+i)
                      enddo
                      close(unit=12)
                   endif

                   call int2str(proc-1,junk)

                   if (ix_multi==1) then
                      open(unit=13,file='DATA/COORDINATES/zco_'//junk(1:len_trim(junk)),action='read')
                      read(13,*) junk
                      read(13,*) nz
                      read(13,*) junk
                      do i=0,nz-1
                         read(13,*) zco(nz_max-i)
                      enddo
                      close(unit=13)
                   endif


                   !=======================================================================
                   ! find index
                   !=======================================================================
                   
                   call int2str(proc-1,junk)

                   open(unit=13,file='DATA/COORDINATES/yco_'//junk(1:len_trim(junk)),action='read')
                   read(13,*) junk
                   read(13,*) ny
                   read(13,*) junk
                   do i=0,ny-1
                      read(13,*) yco(i)
                   enddo
                   close(unit=13)

                   dist=ymax_global
                   index=0
                   
                   do i=0,(ny_max-ny_min)
                      if (abs(yco(i)-val)<dist) then
                         index=i
                         dist=abs(yco(i)-val)
                      endif
                   enddo

                   !write(*,*) proc-1, index
                   
                   !=======================================================================
                   ! make slice
                   !=======================================================================
                   
                   call int2str(proc-1,junk)

                   if (par==1) then
                      open(unit=1001,file='DATA/OUTPUT/grad_rho__'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   elseif (par==2) then
                      open(unit=1001,file='DATA/OUTPUT/grad_lambda_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   elseif (par==3) then
                      open(unit=1001,file='DATA/OUTPUT/grad_mu_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   endif

                   allocate(model(nx_min:nx_max,ny_min:ny_max,nz_min:nz_max),STAT=status)
                   read(1001) model
                   
                   do i=nx_min,nx_max
                      do j=nz_min,nz_max
                         slice(i,j)=model(i,index+ny_min,nz_max-j+nz_min)
                         if (slice(i,j)==0) then
                            write(*,*) 'alert'
                         endif
                      enddo
                   enddo

                   close(unit=1001)
                   deallocate(model)
                   
                   !=======================================================================
                   !=======================================================================



                elseif ((plane==3) .and. (zmin<=val) .and. (zmax>=val)) then
                   
                   !=======================================================================
                   ! make coordinate files
                   !=======================================================================
                   
                   call int2str(proc-1,junk)

                   if (iy_multi==1) then
                      open(unit=12,file='DATA/COORDINATES/xco_'//junk(1:len_trim(junk)),action='read')
                      read(12,*) junk
                      read(12,*) nx
                      read(12,*) junk
                      do i=0,nx-1
                         read(12,*) xco(nx_min+i)
                      enddo
                      close(unit=12)
                   endif

                   call int2str(proc-1,junk)

                   if (ix_multi==1) then
                      open(unit=13,file='DATA/COORDINATES/yco_'//junk(1:len_trim(junk)),action='read')
                      read(13,*) junk
                      read(13,*) ny
                      read(13,*) junk
                      do i=0,ny-1
                         read(13,*) yco(ny_min+i)
                      enddo
                      close(unit=13)
                   endif

                   
                   !=======================================================================
                   ! find index
                   !=======================================================================
                   
                   call int2str(proc-1,junk)

                   open(unit=13,file='DATA/COORDINATES/zco_'//junk(1:len_trim(junk)),action='read')
                   read(13,*) junk
                   read(13,*) nz
                   read(13,*) junk
                   do i=0,nz-1
                      read(13,*) zco(i)
                   enddo
                   close(unit=13)

                   dist=zmax_global
                   index=0
                   
                   do i=0,(nz_max-nz_min)
                      if (abs(zco(i)-val)<dist) then
                         index=i
                         dist=abs(zco(i)-val)
                      endif
                   enddo
                   
                   !=======================================================================
                   ! make slice
                   !=======================================================================

                   call int2str(proc-1,junk)

                   if (par==1) then
                      open(unit=1001,file='DATA/OUTPUT/grad_rho__'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   elseif (par==2) then
                      open(unit=1001,file='DATA/OUTPUT/grad_lambda_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   elseif (par==3) then
                      open(unit=1001,file='DATA/OUTPUT/grad_mu_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit)),&
                           action='read',form='unformatted')
                   endif

                   allocate(model(nx_min:nx_max,ny_min:ny_max,nz_min:nz_max),STAT=status)
                   read(1001) model
                   
                   !write(*,*) maxval(model), minval(model), index

                   slice(nx_min:nx_max,ny_min:ny_max)=model(nx_min:nx_max,ny_min:ny_max,nz_min+index)

                   close(unit=1001)
                   deallocate(model)

                   !=======================================================================
                   !=======================================================================


                endif


			
	enddo
		
	close(UNIT=15)

        !========================================================
        ! write coordinates and slices to files
        !========================================================

        if (plane==1) then

           open(unit=100,file='DATA/OUTPUT/slice_x',action='write')
           open(unit=101,file='DATA/OUTPUT/slice_yco',action='write')
           open(unit=102,file='DATA/OUTPUT/slice_zco',action='write')

           write(101,*) ny_global
           do k=0,ny_global
              write(101,*) yco(k)
           enddo

           write(102,*) nz_global
           do k=0,nz_global
              write(102,*) zco(k)
           enddo

           do k=0,nz_global
              do j=0,ny_global
                 write(100,*) slice(j,k)
              enddo
           enddo

           close(unit=100)
           close(unit=101)
           close(unit=102)

        elseif (plane==2) then

           open(unit=100,file='DATA/OUTPUT/slice_y',action='write')
           open(unit=101,file='DATA/OUTPUT/slice_xco',action='write')
           open(unit=102,file='DATA/OUTPUT/slice_zco',action='write')
           
           write(101,*) nx_global
           do k=0,nx_global
              write(101,*) xco(k)
           enddo

           write(102,*) nz_global
           do k=0,nz_global
              write(102,*) zco(k)
           enddo

           do k=0,nz_global
              do i=0,nx_global
                 write(100,*) slice(i,k)
              enddo
           enddo

           close(unit=100)
           close(unit=101)
           close(unit=102)

        elseif (plane==3) then

           open(unit=100,file='DATA/OUTPUT/slice_z',action='write')
           open(unit=101,file='DATA/OUTPUT/slice_xco',action='write')
           open(unit=102,file='DATA/OUTPUT/slice_yco',action='write')

           write(101,*) nx_global
           do k=0,nx_global
              write(101,*) xco(k)
           enddo

           write(102,*) ny_global
           do k=0,ny_global
              write(102,*) yco(k)
           enddo

           do j=0,ny_global
              do i=0,nx_global
                 write(100,*) slice(i,j)
              enddo
           enddo

           close(unit=100)
           close(unit=101)
           close(unit=102)

        endif

        deallocate(xco)
        deallocate(yco)
        deallocate(zco)

!        deallocate(model)
        deallocate(slice)

end program make_slice_gradient




!==============================================================================
! subroutine for integer to string conversion
!==============================================================================

subroutine int2str(value, string)
implicit none

	integer, intent(in) :: value
	character(len=*), intent(inout) :: string
	
	character(len=10) :: c
	integer :: k, n, new_value, is
	real :: e
	
	e=1e9
	is=0
	
	if (value==0) then
		string(1:1)='0'
		string(2:10)=' '
	else
	
		new_value=value
	
		do k=1,10
			c(k:k)=char(floor(new_value/e)+48)
		
			if ((floor(new_value/e)==0) .and. (is==0)) then
				n=k
			else
				is=1
			endif
		
			new_value=new_value-e*floor(new_value/e)
			e=e/10
			string(k:k)=' '
		
		enddo
		
		string(1:10-n)=c(n+1:10)
		
	endif
	
	if (len(string)>10) then
		string(11:len(string))=' '
	endif

return
end subroutine int2str

